"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var string_character_stream_1 = require("./string-character-stream");
var token_1 = require("./token");
var token_type_1 = require("./token-type");
var SYMBOLS = [
    // Unary
    token_type_1.TokenType.Exclamation,
    token_type_1.TokenType.Plus,
    token_type_1.TokenType.Minus,
    // Boolean
    token_type_1.TokenType.ExclamationEquals,
    token_type_1.TokenType.Equals,
    token_type_1.TokenType.GreaterThan,
    token_type_1.TokenType.GreaterThanEquals,
    token_type_1.TokenType.LessThan,
    token_type_1.TokenType.LessThanEquals,
    token_type_1.TokenType.Ampersand,
    token_type_1.TokenType.Pipe,
    // Multipliers
    token_type_1.TokenType.Slash,
    token_type_1.TokenType.Asterisk,
    token_type_1.TokenType.Percent,
    // Other
    token_type_1.TokenType.Comma,
    token_type_1.TokenType.ParenthesisOpen,
    token_type_1.TokenType.ParenthesisClose,
    token_type_1.TokenType.Period,
];
var DefaultLexer = /** @class */ (function () {
    function DefaultLexer(input) {
        this.numCharRegex = /[0-9]/;
        this.idCharRegex = /[a-zA-Z]/;
        if (this.isCharacterStream(input)) {
            this.stream = input;
        }
        else if (typeof input === 'string') {
            this.stream = new string_character_stream_1.StringCharacterStream(input);
        }
        else {
            throw new Error('Unrecognized input type. input must be of type "CharacterStream | string".');
        }
    }
    DefaultLexer.prototype.isCharacterStream = function (input) {
        return input.getCurrentCharacter;
    };
    DefaultLexer.prototype.peekNextToken = function () {
        if (!this.nextToken) {
            this.nextToken = this.constructNextToken();
        }
        return this.nextToken;
    };
    DefaultLexer.prototype.getNextToken = function () {
        if (this.nextToken) {
            this.currentToken = this.nextToken;
            this.nextToken = null;
        }
        else {
            this.currentToken = this.constructNextToken();
        }
        return this.currentToken;
    };
    DefaultLexer.prototype.parseIdentifier = function () {
        var buffer = this.stream.getCurrentCharacter();
        while (this.idCharRegex.test(this.stream.peekNextCharacter())) {
            buffer += this.stream.getNextCharacter();
        }
        return this.createToken(token_type_1.TokenType.Identifier, buffer);
    };
    DefaultLexer.prototype.parseNumber = function () {
        var buffer = this.stream.getCurrentCharacter();
        var hasDot = false;
        var nextChar = this.stream.peekNextCharacter();
        while (nextChar === '.' || this.numCharRegex.test(nextChar)) {
            if (nextChar === '.') {
                if (hasDot) {
                    break;
                }
                hasDot = true;
            }
            buffer += this.stream.getNextCharacter();
            nextChar = this.stream.peekNextCharacter();
        }
        return this.createToken(token_type_1.TokenType.Number, buffer);
    };
    DefaultLexer.prototype.constructNextToken = function () {
        var curChar;
        var _loop_1 = function () {
            switch (true) {
                case this_1.idCharRegex.test(curChar): return { value: this_1.parseIdentifier() };
                case this_1.numCharRegex.test(curChar): return { value: this_1.parseNumber() };
                case !!SYMBOLS.find(function (s) { return s.startsWith(curChar); }): {
                    var lastFullMatch = void 0;
                    var searchString_1 = curChar;
                    var hasPeeked = false;
                    while (SYMBOLS.find(function (s) { return s.startsWith(searchString_1); })) {
                        if (hasPeeked) {
                            this_1.stream.getNextCharacter();
                        }
                        lastFullMatch = SYMBOLS.find(function (s) { return s === searchString_1; }) || lastFullMatch;
                        searchString_1 += this_1.stream.peekNextCharacter();
                        hasPeeked = true;
                    }
                    return { value: this_1.createToken(lastFullMatch, lastFullMatch) };
                }
                case /\W/.test(curChar):
                    // Ignore whitespace.
                    break;
                default: throw new Error("Unknown token: '" + curChar + "'.");
            }
        };
        var this_1 = this;
        while (curChar = this.stream.getNextCharacter()) {
            var state_1 = _loop_1();
            if (typeof state_1 === "object")
                return state_1.value;
        }
        // Terminator at end of stream.
        return this.createToken(token_type_1.TokenType.Terminator);
    };
    DefaultLexer.prototype.createToken = function (type, value) {
        var position = this.stream.getCurrentPosition();
        if (value) {
            position -= value.length - 1;
        }
        return new token_1.Token(type, position, value);
    };
    return DefaultLexer;
}());
exports.DefaultLexer = DefaultLexer;
//# sourceMappingURL=default-lexer.js.map